//
//  CityWeatherViewController.swift
//  El Vuelo del Grajo
//
//  Created by Santiago Pavón on 7/12/14.
//  Copyright (c) 2014 UPM. All rights reserved.
//

import UIKit
import MapKit


class CityWeatherViewController: UITableViewController {

    // URLs Base de la API y de los iconos.
    fileprivate let API_OPEN_WEATHER_URL = "http://api.openweathermap.org/data/2.5/weather"

    fileprivate let ICON_OPEN_WEATHER_URL = "http://openweathermap.org/img/w"

    // Hay que crearse una cuenta en http://api.openweathermap.org para poder acceder al API.
    fileprivate let APIID = ""

    // Ciudad a mostrar
    var city: String!
    
    @IBOutlet weak var temperatureCell: UITableViewCell!
    @IBOutlet weak var temperatureMaxCell: UITableViewCell!
    @IBOutlet weak var temperatureMinCell: UITableViewCell!
    @IBOutlet weak var humidityCell: UITableViewCell!
    @IBOutlet weak var pressureCell: UITableViewCell!
    @IBOutlet weak var sunsetCell: UITableViewCell!
    @IBOutlet weak var sunriseCell: UITableViewCell!
    @IBOutlet weak var mapView: MKMapView!

    
    @IBOutlet var weatherCells: [UITableViewCell]!
    
    
    @IBOutlet weak var w: UILabel!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        title = city
    }
    
    
    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(animated)
        
        checkAPIID()
        downloadWeather()
    }
    
    fileprivate func checkAPIID() {
        if APIID == "" {
            let alert = UIAlertController(title: "Regístrese", message: "Para utilizar esta aplicación debe registrarse en http://openweathermap.org y obtener una clase de acceso al API", preferredStyle: .alert)
            
            alert.addAction(UIAlertAction(
                title: "Vale",
                style: UIAlertActionStyle.default) {(_: UIAlertAction) -> Void in
                    self.dismiss(animated: true, completion: nil)
                })
            
            present(alert, animated: true, completion: nil)
        }
    }
    
    
    
    // Descargar datos usando NSData(contentsOfURL: url)
    fileprivate func downloadWeather() {
        
        UIApplication.shared.isNetworkActivityIndicatorVisible = true
        defer {
            UIApplication.shared.isNetworkActivityIndicatorVisible = false
        }
        
        guard let query = "q=\(city!)&units=metric&lang=sp&APPID=\(APIID)".addingPercentEncoding(withAllowedCharacters: .urlQueryAllowed) else {
            print("Error escapando URL")
            return
        }
        
        guard let url = URL(string: "\(API_OPEN_WEATHER_URL)?\(query)") else {
            print("Error: URL mal formada")
            return
        }
        
        print("URL = \(url)")
        
        guard let data = try? Data(contentsOf: url) else {
            print("Error: no se han podido descargar los datos.")
            return
        }

        let decoder = JSONDecoder()
        do {
            let wm = try decoder.decode(WeatherMap.self, from: data)
            print(wm)
            
            if wm.cod == 200 {
                updateOutlets(wm)
            } else {
                print(wm.message ?? "Error: Desconocido")
            }
        } catch {
            print("ERROR: No puedo sacar el JSON:", error.localizedDescription)
            if let strData = String(data: data, encoding: .utf8) {
                print("JSON = \(strData)")
            }
        }
    }


    fileprivate func getIconNamed(_ name: String) -> UIImage? {
        
        UIApplication.shared.isNetworkActivityIndicatorVisible = true
        defer {
            UIApplication.shared.isNetworkActivityIndicatorVisible = false
        }
        
        guard let url = URL(string: "\(ICON_OPEN_WEATHER_URL)/\(name).png") else {
            print("Error: URL mal formada")
            return nil
        }
        
        guard let data = try? Data(contentsOf: url) else {
            print("Error: no se han podido descargar el icono.")
            return nil
        }
        
        guard let img = UIImage(data: data) else {
            print("Los datos recibidos no tienen un formato inesperado.")
            return nil
        }
        
        return img
    }
    
    fileprivate func updateOutlets(_ wm: WeatherMap) {
        
        let center = CLLocationCoordinate2D(latitude: CLLocationDegrees(wm.coord.lat), longitude: CLLocationDegrees(wm.coord.lon))
        let span = MKCoordinateSpanMake(0.5, 0.5)
        let reg =  MKCoordinateRegionMake(center, span)
        mapView.setRegion(reg, animated: false)
        mapView.mapType = .hybrid
        mapView.isUserInteractionEnabled = false
      
        temperatureCell.detailTextLabel?.text = "\(wm.main.temp)ºC"
        temperatureMaxCell.detailTextLabel?.text = "\(wm.main.temp_max)ºC"
        temperatureMinCell.detailTextLabel?.text = "\(wm.main.temp_min)ºC"
        humidityCell.detailTextLabel?.text = "\(wm.main.humidity)%"
        pressureCell.detailTextLabel?.text = "\(wm.main.pressure)P"
        
        let dateFormatter = DateFormatter()
        dateFormatter.dateFormat = "HH:mm"
        let datess = Date(timeIntervalSince1970: wm.sys.sunrise)
        sunriseCell.detailTextLabel?.text = dateFormatter.string(from: datess)
        let datesr = Date(timeIntervalSince1970: wm.sys.sunset)
        sunsetCell.detailTextLabel?.text = dateFormatter.string(from: datesr)

        for i in 0..<5 {
            weatherCells[i].textLabel?.text  = wm.weather.count > i ? wm.weather[i].description : ""
            weatherCells[i].imageView?.image = wm.weather.count > i ? getIconNamed(wm.weather[i].icon) : nil
        }
    }

}
